]> git.sesse.net Git - backup.sh/blobdiff - backup.sh
Take backup at lowest possible (non-idle) CPU priority, and idle I/O priority.
[backup.sh] / backup.sh
index 18968c8202ab309afc8a7f5f0567b8bffe477dfa..67a9e1ecc18f0b7c4bf5b6a949ec314a952ac01a 100755 (executable)
--- a/backup.sh
+++ b/backup.sh
 # o $R has no proper meaning because of the forking subshell stuff. We need a
 #   more robust way of returning non-zero on errors, which we probably don't
 #   right now.
-# o The code around line 211 and downwards is messy.
 #
 
 # Configuration
 LOCKFILE=/home/backup/backuprun.lockfile
-confdir=/home/backup/conf/           # Configuration files
-storagedir=/backup                   # Where we keep backups
-maksantallfulle=3                    # Number of full backups
-dagermellomfulle=30                  # Days between full backups
-DATE=`date "+%Y%m%d%H%M"`            #format: touch
-DATEs=`date "+%Y-%m-%d %H:%M"`       #format: tar
+confdir=/home/backup/conf/              # Configuration files
+storagedir=/backup                      # Where we keep backups
+maksantallfulle=3                       # Number of full backups
+dagermellomfulle=30                     # Days between full backups
+dagermellomfulleforfast=32              # Days between full backups for machines with fixed
+                                        # full-backup date (in case the fixed day gets missed)
+DAY_OF_MONTH=`date "+%d" | sed s/^0//`  # from 1 to 31
+DATE=`date "+%Y%m%d%H%M"`               #format: touch
+DATEs=`date "+%Y-%m-%d %H:%M"`          #format: tar
 
 # Exclude-pattern
 exclude=$confdir/exclude
@@ -63,12 +65,12 @@ trap die SIGINT SIGTERM
 
 # Don't start if we're already running
 if [ -e $LOCKFILE ]; then
-       echo `date`": $LOCKFILE exists, exiting."
-       echo `date`": $LOCKFILE exists, exiting." >&2
+       echo `date`": $LOCKFILE exists for pid `cat $LOCKFILE`, exiting."
+       echo `date`": $LOCKFILE exists for pid `cat $LOCKFILE`, exiting." >&2
        exit 1
 fi;
 touch $LOCKFILE
-echo "$$" > $LOCKFILE
+echo $$ > $LOCKFILE
 
 #syntax of remotestat:
 #hostname:/directory/
@@ -101,7 +103,7 @@ backup()
 {
 
  echo -n `date` >&2
- printf " $computer: $computer:$filesystem $backuplevel backup" >&2
+ printf " $computer: $computer:$filesystem $backuplevel backup\n" >&2
 
  if [ "$backuplevel" = "daglig" ] || [ "$backuplevel" = "incremental" ]
  then
@@ -111,14 +113,6 @@ backup()
   lastcmd=""
  fi
 
- #if this client has a special tar
- #we need to find a better solution to this conf-issue
- if [ -f $confdir/tar.$computer ] ; then
-   tar=`cat $confdir/tar.$computer`
- else 
-   tar=tar
- fi
-
  # expand the exclude-path for use with tar
  exf=`ssh root@$computer "ls ~/.backup/exclude"`
 
@@ -133,7 +127,7 @@ backup()
  #    Pipe the stuff over ssh to ourselves, run pee to cat the contents to a
  #    file and build a file list, respectivly.  
  TARFILE=$DATE.tmp
- TARCMD="ssh root@$computer \"$tar --one-file-system -zcf - -C $filesystem . $lastcmd \
+ TARCMD="ssh root@$computer \"nice -n 19 ionice -c3 tar --one-file-system -zcf - -C $filesystem . $lastcmd \
        --exclude-from=$exf\" | pee \"cat > $TARFILE\" \"tar tzvf -\""
  echo `date`" $computer: Running $TARCMD" >&2
  eval $TARCMD > $DATE.idx
@@ -213,21 +207,20 @@ do
  fi
 
  # Try to copy preeexec and postexec if they exist
-# TODO: Gah, clean this mess!
- [ -f $confdir/preexec.$computer ] && (
-    scp $confdir/preexec.$computer  root@$computer:~/.backup/preexec >&2 ||
-     ( echo `date`" $computer: Could not copy preexec.$computer to $computer:~/.backup/preexec - skipping backup of $computer"
-       R=1
-       continue
-     )
-    )
- [ -f $confdir/postexec.$computer ] && (
-    scp $confdir/postexec.$computer root@$computer:~/.backup/postexec >&2 ||
-     ( echo `date`" $computer: Could not copy postexec.$computer to $computer:~/.backup/postexec - skipping backup of $computer"
-       R=1
-       continue
-     )
-    )
+ if [ -f $confdir/preexec.$computer ]; then
+   if ! scp $confdir/preexec.$computer  root@$computer:~/.backup/preexec >&2; then
+     echo `date`" $computer: Could not copy preexec.$computer to $computer:~/.backup/preexec - skipping backup of $computer"
+     R=1
+     continue
+   fi
+ fi
+ if [ -f $confdir/postexec.$computer ]; then
+   if ! scp $confdir/postexec.$computer root@$computer:~/.backup/postexec >&2; then
+     echo `date`" $computer: Could not copy postexec.$computer to $computer:~/.backup/postexec - skipping backup of $computer"
+     R=1
+     continue
+   fi
+ fi
 
  # Try to run preexec if it exist
  if ! ssh root@$computer "[ ! -f ~/.backup/preexec ] || /bin/bash -x ~/.backup/preexec" >&2; then
@@ -257,11 +250,26 @@ do
    echo $DATEs > $storagedir/$computer/$sfilesystem/.date
   fi
 
-  # Check if we want a full backup
-  if [ -z "`find $storagedir/$computer/$sfilesystem/full/ -name \*tgz -mtime -$dagermellomfulle`" ]; then
+  # Check if we want a full backup - Debug statement that causes noise in cron mail:
+  # printf " $computer $filesystem: $confdir/fastfullbackupdag.$computer.$sfilesystem\n"
+  if [ -f $confdir/fastfullbackupdag.$computer.$sfilesystem ]; then
+    fullbackup_min_for_this_machine=$dagermellomfulleforfast
+    if [ "$DAY_OF_MONTH" = "`cat $confdir/fastfullbackupdag.$computer.$sfilesystem`" ]; then
+      backuplevel=full
+    fi
+  elif [ -f $confdir/fastfullbackupdag.$computer ]; then
+    fullbackup_min_for_this_machine=$dagermellomfulleforfast
+    if [ "$DAY_OF_MONTH" = "`cat $confdir/fastfullbackupdag.$computer`" ]; then
+      backuplevel=full
+    fi
+  else
+    fullbackup_min_for_this_machine=$dagermellomfulle 
+  fi
+
+  if [ -z "`find $storagedir/$computer/$sfilesystem/full/ -name \*tgz -mtime -$fullbackup_min_for_this_machine`" ]; then
    backuplevel=full
   fi
+
   # We want to be here
   cd $storagedir/$computer/$sfilesystem/$backuplevel || die